/*
    conn.c: Handles tcp and udp connections.
    Part of the EasyTcpIp library.
    Copyright (C) 1999 by Marcelo Gornstein

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    see "COPYING" file for details.

If you want to contact me:
Marcelo Gornstein
mgornstein@usa.net
*/
#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "tcpip.h"

int openserver (struct host *h, int protocol, int backlog) {
    int r;
    h->sockin.sin_family = AF_INET;
    h->sockin.sin_addr.s_addr = htonl (INADDR_ANY);
    h->sockin.sin_port = htons (h->port);
    h->protocol = protocol;
    if (protocol == IPPROTO_TCP) h->s = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if (protocol == IPPROTO_UDP) h->s = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if (h->s < 0) return (-1);
    r = bind (h->s, (struct sockaddr *) (&h->sockin), sizeof (struct sockaddr));
    if (r < 0) return (-1);
    r = listen (h->s, backlog);
    if (r < 0) return (-1);
    return (0);
}

struct host *getclient (struct host *s) {
    int r;
    struct host *c = &hst;
    memset ((void *) (c), '\0', sizeof (struct host));
    r = sizeof (struct sockaddr);
    c->s = accept (s->s, (struct sockaddr *) (&c->sockin), &r);
    if (c->s < 0) return (0);
    strcpy (c->ip, inet_ntoa (c->sockin.sin_addr));
    r = (int) gethostinfo (c->ip);
    if (!r) return (0);
    memcpy ((void *) (c), (void *) (r), sizeof (struct host));
    if (s->protocol == IPPROTO_TCP) {
	s->tcpconnects++;
	c->tcpconnects++;
    }
    if (s->protocol == IPPROTO_UDP) {
	s->udpconnects++;
	c->udpconnects++;
    }
    return (c);
}

int openconn (struct host *h, int protocol) {
    int r;
    h->protocol = protocol;    
    h->sockin.sin_family = AF_INET;
    h->sockin.sin_port = htons (h->port);
    if (protocol == IPPROTO_UDP) {
	if (h->udpconnects) return (-1);
	h->s = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if (h->s < 0) return (-1);
	r = connect (h->s, (struct sockaddr *) (&h->sockin), sizeof (struct sockaddr));
	if (r < 0) return (-1);
	h->udpconnects++;
	return (0);
    } else if (protocol == IPPROTO_TCP) {
	if (h->tcpconnects) return (-1);
	h->s = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (h->s < 0) return (-1);
	r = connect (h->s, (struct sockaddr *) (&h->sockin), sizeof (struct sockaddr));
	if (r < 0) return (-1);
	h->tcpconnects++;
	return (0);
    }
    return (-1);
}
int closeconn (struct host *h) {
    int r;
    r = close (h->s);
    if (r < 0) return (-1);
    if (h->protocol == IPPROTO_UDP) h->udpconnects--;
    if (h->protocol == IPPROTO_TCP) h->tcpconnects--;
    return (0);
}